@@ -2,32 +2,32 @@
 # created by Tobias Powalowski <tpowa@archlinux.org>
 # grub-uefi related commands have been copied from grub-mkstandalone and grub-mkrescue scripts in extra/grub2-common package
 
-WD="${PWD}/"
+[[ -z "${_DO_x86_64}" ]] && _DO_x86_64="1"
+[[ -z "${_DO_i686}" ]] && _DO_i686="1"
 
-APPNAME="$(basename "${0}")"
+[[ -z "${WD}" ]] && WD="${PWD}/"
 
-_DO_x86_64="1"
-_DO_i686="1"
+_BASENAME="$(basename "${0}")"
 
 usage () {
-    echo "${APPNAME}: usage"
-    echo "CREATE ALLINONE USB/CD IMAGES"
-    echo "-----------------------------"
-    echo "Run in archboot x86_64 chroot first ..."
-    echo "create-allinone.sh -t"
-    echo "Run in archboot 686 chroot then ..."
-    echo "create-allinone.sh -t"
-    echo "Copy the generated tarballs to your favorite directory and run:"
-    echo "${APPNAME} -g <any other option>"
-    echo ""
-    echo "PARAMETERS:"
-    echo "  -g                  Start generation of images."
-    echo "  -i=IMAGENAME        Your IMAGENAME."
-    echo "  -r=RELEASENAME      Use RELEASENAME in boot message."
-    echo "  -k=KERNELNAME       Use KERNELNAME in boot message."
-    echo "  -lts=LTSKERNELNAME  Use LTSKERNELNAME in boot message."
-    echo "  -h                  This message."
-    exit 0
+	echo "${_BASENAME}: usage"
+	echo "CREATE ALLINONE USB/CD IMAGES"
+	echo "-----------------------------"
+	echo "Run in archboot x86_64 chroot first ..."
+	echo "create-allinone.sh -t"
+	echo "Run in archboot 686 chroot then ..."
+	echo "create-allinone.sh -t"
+	echo "Copy the generated tarballs to your favorite directory and run:"
+	echo "${_BASENAME} -g <any other option>"
+	echo ""
+	echo "PARAMETERS:"
+	echo "  -g                  Start generation of images."
+	echo "  -i=IMAGENAME        Your IMAGENAME."
+	echo "  -r=RELEASENAME      Use RELEASENAME in boot message."
+	echo "  -k=KERNELNAME       Use KERNELNAME in boot message."
+	echo "  -lts=LTSKERNELNAME  Use LTSKERNELNAME in boot message."
+	echo "  -h                  This message."
+	exit 0
 }
 
 [[ -z "${1}" ]] && usage
@@ -36,6 +36,7 @@ ALLINONE_PRESET="/etc/archboot/presets/allinone"
 ALLINONE_LTS_PRESET="/etc/archboot/presets/allinone-lts"
 TARBALL_HELPER="/usr/bin/archboot-tarball-helper.sh"
 USBIMAGE_HELPER="/usr/bin/archboot-usbimage-helper.sh"
+UPDATEISO_HELPER="/usr/bin/archboot-update-iso.sh"
 
 # change to english locale!
 export LANG="en_US"
@@ -73,9 +74,23 @@ fi
 # set defaults, if nothing given
 [[ -z "${KERNEL}" ]] && KERNEL="$(uname -r)"
 [[ -z "${LTS_KERNEL}" ]] && LTS_KERNEL="$(cat /lib/modules/extramodules-3.0-lts/version)"
-[[ -z "${RELEASENAME}" ]] && RELEASENAME="2k12-R1"
+[[ -z "${RELEASENAME}" ]] && RELEASENAME="2k12-R2"
 [[ -z "${IMAGENAME}" ]] && IMAGENAME="Archlinux-allinone-$(date +%Y.%m)"
 
+IMAGENAME_OLD="${IMAGENAME}"
+
+if [[ "${_DO_x86_64}" == "1" ]] && [[ "${_DO_i686}" == "1" ]]; then
+	IMAGENAME="${IMAGENAME}-dual"
+fi
+
+if [[ "${_DO_x86_64}" == "1" ]] && [[ "${_DO_i686}" != "1" ]]; then
+	IMAGENAME="${IMAGENAME}-x86_64"
+fi
+
+if [[ "${_DO_x86_64}" != "1" ]] && [[ "${_DO_i686}" == "1" ]]; then
+	IMAGENAME="${IMAGENAME}-i686"
+fi
+
 ALLINONE="$(mktemp -d /tmp/allinone.XXX)"
 
 # create directories
@@ -93,7 +108,7 @@ _merge_initramfs_files() {
 		bsdtar xf "${CORE64_LTS}/tmp"/*/boot/initrd.img
 		
 		cd  "${CORE64}/tmp/initrd"
-		find . -print0 | bsdcpio -0oH newc | xz --check=crc32 --lzma2=dict=1MiB > "${CORE64}/tmp/initramfs_x86_64.img"
+		find . -print0 | bsdcpio -0oH newc | lzma > "${CORE64}/tmp/initramfs_x86_64.img"
 	fi
 	
 	if [[ "${_DO_i686}" == "1" ]]; then
@@ -104,7 +119,7 @@ _merge_initramfs_files() {
 		bsdtar xf "${CORE_LTS}/tmp"/*/boot/initrd.img
 		
 		cd  "${CORE}/tmp/initrd"
-		find . -print0 | bsdcpio -0oH newc | xz --check=crc32 --lzma2=dict=1MiB > "${CORE}/tmp/initramfs_i686.img"
+		find . -print0 | bsdcpio -0oH newc | lzma > "${CORE}/tmp/initramfs_i686.img"
 	fi
 	
 	cd "${WD}/"
@@ -140,7 +155,7 @@ _prepare_packages() {
 		tar xvf core-x86_64.tar -C "${CORE64}" || exit 1
 		tar xvf core-lts-x86_64.tar -C "${CORE64_LTS}" || exit 1
 		
-		cp -r "${CORE64_LTS}/tmp"/*/core-x86_64 "${PACKAGES_TEMP_DIR}/core-x86_64"
+		cp -rf "${CORE64_LTS}/tmp"/*/core-x86_64 "${PACKAGES_TEMP_DIR}/core-x86_64"
 		rm -rf "${CORE64_LTS}/tmp"/*/core-x86_64
 		mksquashfs "${PACKAGES_TEMP_DIR}/core-x86_64/" "${PACKAGES_TEMP_DIR}/archboot_packages_x86_64.squashfs" -comp xz -noappend -all-root
 		mv "${PACKAGES_TEMP_DIR}/archboot_packages_x86_64.squashfs" "${ALLINONE}/packages/"
@@ -153,14 +168,14 @@ _prepare_packages() {
 		tar xvf core-i686.tar -C "${CORE}" || exit 1
 		tar xvf core-lts-i686.tar -C "${CORE_LTS}" || exit 1
 		
-		cp -r "${CORE_LTS}/tmp"/*/core-i686 "${PACKAGES_TEMP_DIR}/core-i686"
+		cp -rf "${CORE_LTS}/tmp"/*/core-i686 "${PACKAGES_TEMP_DIR}/core-i686"
 		rm -rf "${CORE_LTS}/tmp"/*/core-i686
 		mksquashfs "${PACKAGES_TEMP_DIR}/core-i686/" "${PACKAGES_TEMP_DIR}/archboot_packages_i686.squashfs" -comp xz -noappend -all-root
 		mv "${PACKAGES_TEMP_DIR}/archboot_packages_i686.squashfs" "${ALLINONE}/packages/"
 	fi
 	
 	# move in 'any' packages
-	cp -r "${CORE_LTS}/tmp"/*/core-any "${PACKAGES_TEMP_DIR}/core-any"
+	cp -rf "${CORE_LTS}/tmp"/*/core-any "${PACKAGES_TEMP_DIR}/core-any"
 	rm -rf "${CORE_LTS}/tmp"/*/core-any
 	mksquashfs "${PACKAGES_TEMP_DIR}/core-any/" "${PACKAGES_TEMP_DIR}/archboot_packages_any.squashfs" -comp xz -noappend -all-root
 	
@@ -188,13 +203,22 @@ _prepare_other_files() {
 
 _download_uefi_shell_tianocore() {
 	
-	mkdir -p "${ALLINONE}/efi/shell/"
+	mkdir -p "${ALLINONE}/efi/tools/"
 	
 	## Download Tianocore UDK/EDK2 ShellBinPkg UEFI "Full Shell" - For UEFI Spec. >=2.3 systems
-	curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${ALLINONE}/efi/shell/shellx64.efi" "https://edk2.svn.sourceforge.net/svnroot/edk2/trunk/edk2/ShellBinPkg/UefiShell/X64/Shell.efi"
+	curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${ALLINONE}/efi/tools/shellx64.efi" "https://edk2.svn.sourceforge.net/svnroot/edk2/trunk/edk2/ShellBinPkg/UefiShell/X64/Shell.efi"
 	
 	## Download Tianocore UDK/EDK2 EdkShellBinPkg UEFI "Full Shell" - For UEFI Spec. <2.3 systems
-	curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${ALLINONE}/efi/shell/shellx64_old.efi" "https://edk2.svn.sourceforge.net/svnroot/edk2/trunk/edk2/EdkShellBinPkg/FullShell/X64/Shell_Full.efi"
+	curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${ALLINONE}/efi/tools/shellx64_old.efi" "https://edk2.svn.sourceforge.net/svnroot/edk2/trunk/edk2/EdkShellBinPkg/FullShell/X64/Shell_Full.efi"
+	
+}
+
+_download_uefi_refind_sourceforge() {
+	
+	mkdir -p "${ALLINONE}/packages/"
+	
+	## Download latest rEFInd bin archive from sourceforge
+	curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${ALLINONE}/packages/refind-bin.zip" -L "http://sourceforge.net/projects/refind/files/latest/download"
 	
 }
 
@@ -231,7 +255,7 @@ source "(\${archboot})/boot/grub/grub_archboot.cfg"
 
 EOF
 	
-	cp "${ALLINONE}/boot/grub/grub_standalone_archboot.cfg" "${ALLINONE}/boot/grub/grub.cfg"
+	cp -f "${ALLINONE}/boot/grub/grub_standalone_archboot.cfg" "${ALLINONE}/boot/grub/grub.cfg"
 	
 	__WD="${PWD}/"
 	
@@ -244,7 +268,7 @@ EOF
 	rm -f "${ALLINONE}/boot/grub/grub.cfg"
 	
 	mkdir -p "${ALLINONE}/efi/boot/"
-	cp "${grub_uefi_mp}/efi/boot/boot${_SPEC_UEFI_ARCH}.efi" "${ALLINONE}/efi/boot/boot${_SPEC_UEFI_ARCH}.efi"
+	cp -f "${grub_uefi_mp}/efi/boot/boot${_SPEC_UEFI_ARCH}.efi" "${ALLINONE}/efi/boot/boot${_SPEC_UEFI_ARCH}.efi"
 	
 	unset _UEFI_ARCH
 	unset _SPEC_UEFI_ARCH
@@ -282,7 +306,7 @@ _prepare_grub_uefi_iso_files() {
 	losetup --detach "${LOOP_DEVICE}"
 	
 	mkdir -p "${ALLINONE}/boot/grub/fonts"
-	cp "/usr/share/grub/unicode.pf2" "${ALLINONE}/boot/grub/fonts/"
+	cp -f "/usr/share/grub/unicode.pf2" "${ALLINONE}/boot/grub/fonts/"
 	
 	mkdir -p "${ALLINONE}/boot/grub/locale/"
 	
@@ -370,6 +394,11 @@ insmod search_fs_file
 insmod linux
 insmod chain
 
+EOF
+	
+	if [[ "${_DO_x86_64}" == "1" ]]; then
+		cat << EOF >> "${ALLINONE}/boot/grub/grub_archboot.cfg"
+
 if [ cpuid -l ]; then
 
     menuentry "Arch Linux (x86_64) archboot" {
@@ -388,6 +417,12 @@ if [ cpuid -l ]; then
 
 fi
 
+EOF
+	fi
+	
+	if [[ "${_DO_i686}" == "1" ]]; then
+		cat << EOF >> "${ALLINONE}/boot/grub/grub_archboot.cfg"
+
 menuentry "Arch Linux (i686) archboot" {
     set gfxpayload="keep"
     set root="\${archboot}"
@@ -402,16 +437,21 @@ menuentry "Arch Linux LTS (i686) archboot" {
     initrd /boot/initramfs_i686.img
 }
 
+EOF
+	fi
+	
+	cat << EOF >> "${ALLINONE}/boot/grub/grub_archboot.cfg"
+
 if [ "\${grub_platform}" == "efi" ]; then
 
     menuentry "UEFI \${_UEFI_ARCH} Shell 2.0 - For Spec. Ver. >=2.3 systems" {
         set root="\${archboot}"
-        chainloader /efi/shell/shell\${_SPEC_UEFI_ARCH}.efi
+        chainloader /efi/tools/shell\${_SPEC_UEFI_ARCH}.efi
     }
 
     menuentry "UEFI \${_UEFI_ARCH} Shell 1.0 - For Spec. Ver. <2.3 systems" {
         set root="\${archboot}"
-        chainloader /efi/shell/shell\${_SPEC_UEFI_ARCH}_old.efi
+        chainloader /efi/tools/shell\${_SPEC_UEFI_ARCH}_old.efi
     }
 
 fi
@@ -430,6 +470,8 @@ _prepare_kernel_initramfs_files
 
 _download_uefi_shell_tianocore
 
+_download_uefi_refind_sourceforge
+
 _prepare_grub_uefi_iso_files
 
 # place syslinux files
@@ -438,6 +480,8 @@ mv "${CORE}/tmp"/*/boot/syslinux/* "${ALLINONE}/boot/syslinux/"
 # Change parameters in boot.msg
 sed -i -e "s/@@DATE@@/$(date)/g" -e "s/@@KERNEL@@/$KERNEL/g"  -e "s/@@LTS_KERNEL@@/$LTS_KERNEL/g" -e "s/@@RELEASENAME@@/$RELEASENAME/g" -e "s/@@BOOTLOADER@@/ISOLINUX/g" "${ALLINONE}/boot/syslinux/boot.msg"
 
+cd "${WD}/"
+
 ## Generate the BIOS+UEFI+ISOHYBRID ISO image using xorriso (extra/libisoburn package) in mkisofs emulation mode
 echo "Generating ALLINONE hybrid ISO ..."
 xorriso -as mkisofs \
@@ -446,25 +490,37 @@ xorriso -as mkisofs \
         -omit-version-number -allow-leading-dots \
         -relaxed-filenames -allow-lowercase -allow-multidot \
         -volid "ARCHBOOT" \
-        -p "prepared by ${APPNAME}" \
+        -preparer "prepared by ${_BASENAME}" \
         -eltorito-boot boot/syslinux/isolinux.bin \
         -eltorito-catalog boot/syslinux/boot.cat \
         -no-emul-boot -boot-load-size 4 -boot-info-table \
         -eltorito-alt-boot --efi-boot boot/grub/grub_uefi_x86_64.bin -no-emul-boot \
         -isohybrid-mbr /usr/lib/syslinux/isohdpfx.bin \
-        -output "${IMAGENAME}.iso" "${ALLINONE}/" > /dev/null 2>&1
+        -output "${IMAGENAME}.iso" "${ALLINONE}/" &> "/tmp/archboot_allinone_xorriso.log"
 
-# cleanup isolinux and migrate to syslinux
-echo "Generating ALLINONE IMG ..."
-rm -f "${ALLINONE}/boot/syslinux/isolinux.bin"
-# Change parameters in boot.msg
-sed -i -e "s/@@DATE@@/$(date)/g" -e "s/@@KERNEL@@/$KERNEL/g" -e "s/@@LTS_KERNEL@@/$LTS_KERNEL/g" -e "s/@@RELEASENAME@@/$RELEASENAME/g" -e "s/@@BOOTLOADER@@/SYSLINUX/g" "${ALLINONE}/boot/syslinux/boot.msg"
+## cleanup isolinux and migrate to syslinux
+# echo "Generating ALLINONE IMG ..."
+# rm -f "${ALLINONE}/boot/syslinux/isolinux.bin"
 
-"${USBIMAGE_HELPER}" "${ALLINONE}" "${IMAGENAME}.img" > /dev/null 2>&1
+## Change parameters in boot.msg
+# sed -i -e "s/@@DATE@@/$(date)/g" -e "s/@@KERNEL@@/$KERNEL/g" -e "s/@@LTS_KERNEL@@/$LTS_KERNEL/g" -e "s/@@RELEASENAME@@/$RELEASENAME/g" -e "s/@@BOOTLOADER@@/SYSLINUX/g" "${ALLINONE}/boot/syslinux/boot.msg"
 
-#create sha256sums.txt
-rm -f sha256sums.txt || true
-sha256sum "${IMAGENAME}.iso" "${IMAGENAME}.img" > sha256sums.txt
+# "${USBIMAGE_HELPER}" "${ALLINONE}" "${IMAGENAME}.img" > /dev/null 2>&1
+
+if [[ -e "${WD}/${IMAGENAME_OLD}-dual.iso" ]] && [[ ! -e "${WD}/${IMAGENAME_OLD}-x86_64.iso" ]]; then
+	_REMOVE_i686="1" _REMOVE_x86_64="0" _UPDATE_SETUP="0" _UPDATE_UEFI_SHELL="0" _UPDATE_UEFI_REFIND="0" _UPDATE_SYSLINUX="0" _UPDATE_SYSLINUX_CONFIG="1" _UPDATE_GRUB_UEFI="0" _UPDATE_GRUB_UEFI_CONFIG="1" "${UPDATEISO_HELPER}" "${WD}/${IMAGENAME_OLD}-dual.iso"
+	mv "${WD}/${IMAGENAME_OLD}-dual-updated-x86_64.iso" "${WD}/${IMAGENAME_OLD}-x86_64.iso"
+fi
+
+if [[ -e "${WD}/${IMAGENAME_OLD}-dual.iso" ]] && [[ ! -e "${WD}/${IMAGENAME_OLD}-i686.iso" ]]; then
+	_REMOVE_i686="0" _REMOVE_x86_64="1" _UPDATE_SETUP="0" _UPDATE_UEFI_SHELL="0" _UPDATE_UEFI_REFIND="0" _UPDATE_SYSLINUX="0" _UPDATE_SYSLINUX_CONFIG="1" _UPDATE_GRUB_UEFI="0" _UPDATE_GRUB_UEFI_CONFIG="1" "${UPDATEISO_HELPER}" "${WD}/${IMAGENAME_OLD}-dual.iso"
+	mv "${WD}/${IMAGENAME_OLD}-dual-updated-i686.iso" "${WD}/${IMAGENAME_OLD}-i686.iso"
+fi
+
+## create sha256sums.txt
+cd "${WD}/"
+rm -f "${WD}/sha256sums.txt" || true
+sha256sum *.iso *.img > "${WD}/sha256sums.txt"
 
 # cleanup
 rm -rf "${grub_uefi_mp}"
