@@ -2,13 +2,20 @@
 # Script for updating existing Archboot iso with newer UEFI shell, GRUB UEFI, and/or /arch/setup script in the initramfs files
 # Contributed by "Keshav P R" <the.ridikulus.rat aatt geemmayil ddoott ccoomm>
 
-_UPDATE_SYSLINUX="1"
-_UPDATE_UEFI_SHELL="1"
-_UPDATE_GRUB_UEFI="1"
-_UPDATE_SETUP="1"
+[[ -z "${_REMOVE_i686}" ]] && _REMOVE_i686="0"
+[[ -z "${_REMOVE_x86_64}" ]] && _REMOVE_x86_64="0"
 
-_REMOVE_i686="1"
-_REMOVE_x86_64="0"
+[[ -z "${_UPDATE_SETUP}" ]] && _UPDATE_SETUP="1"
+[[ -z "${_UPDATE_UEFI_SHELL}" ]] && _UPDATE_UEFI_SHELL="1"
+[[ -z "${_UPDATE_UEFI_REFIND}" ]] && _UPDATE_UEFI_REFIND="1"
+
+[[ -z "${_UPDATE_SYSLINUX}" ]] && _UPDATE_SYSLINUX="1"
+[[ -z "${_UPDATE_SYSLINUX_CONFIG}" ]] && _UPDATE_SYSLINUX_CONFIG="1"
+[[ -z "${_UPDATE_GRUB_UEFI}" ]] && _UPDATE_GRUB_UEFI="1"
+[[ -z "${_UPDATE_GRUB_UEFI_CONFIG}" ]] && _UPDATE_GRUB_UEFI_CONFIG="1"
+
+[[ "${_UPDATE_SYSLINUX}" == "1" ]] && _UPDATE_SYSLINUX_CONFIG="1"
+[[ "${_UPDATE_GRUB_UEFI}" == "1" ]] && _UPDATE_GRUB_UEFI_CONFIG="1"
 
 #############################
 
@@ -21,9 +28,24 @@ _ARCHBOOT_ISO_OLD_NAME="$(basename "${_ARCHBOOT_ISO_OLD_PATH}" | sed 's|\.iso||g
 
 _ARCHBOOT_ISO_EXT_DIR="$(mktemp -d /tmp/archboot_iso_ext.XXXXXXXXXX)"
 
-_ARCHBOOT_ISO_UPDATED_NAME="${_ARCHBOOT_ISO_OLD_NAME}_updated"
+#############################
+
+if [[ "${_REMOVE_x86_64}" != "1" ]] && [[ "${_REMOVE_i686}" != "1" ]]; then
+	_ARCHBOOT_ISO_UPDATED_NAME="${_ARCHBOOT_ISO_OLD_NAME}-updated-dual"
+fi
+
+if [[ "${_REMOVE_x86_64}" != "1" ]] && [[ "${_REMOVE_i686}" == "1" ]]; then
+	_ARCHBOOT_ISO_UPDATED_NAME="${_ARCHBOOT_ISO_OLD_NAME}-updated-x86_64"
+fi
+
+if [[ "${_REMOVE_x86_64}" == "1" ]] && [[ "${_REMOVE_i686}" != "1" ]]; then
+	_ARCHBOOT_ISO_UPDATED_NAME="${_ARCHBOOT_ISO_OLD_NAME}-updated-i686"
+fi
+
 _ARCHBOOT_ISO_UPDATED_PATH="${_ARCHBOOT_ISO_WD}/${_ARCHBOOT_ISO_UPDATED_NAME}.iso"
 
+#############################
+
 echo
 
 if [[ -z "${1}" ]]; then
@@ -32,7 +54,7 @@ if [[ -z "${1}" ]]; then
 	echo
 	echo "Example: ${_BASENAME} /home/user/Desktop/archlinux-2012.01-1-archboot.iso"
 	echo
-	echo "Updated iso will be saves at /home/user/Desktop/archlinux-2012.01-1-archboot_updated.iso "
+	echo "Updated iso will be saved at /home/user/Desktop/archlinux-2012.01-1-archboot_updated.iso "
 	echo "(for example)."
 	echo
 	echo "This script should be run as root user."
@@ -56,14 +78,14 @@ cd "${_ARCHBOOT_ISO_EXT_DIR}/"
 echo
 
 ## Extract the archboot iso using bsdtar
-bsdtar xf "${_ARCHBOOT_ISO_OLD_PATH}"
+bsdtar -C "${_ARCHBOOT_ISO_EXT_DIR}/" -xf "${_ARCHBOOT_ISO_OLD_PATH}"
 # 7z x "${_ARCHBOOT_ISO_OLD_PATH}"
 echo
 
 rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/[BOOT]/" || true
 echo
 
-[[ -e "${_ARCHBOOT_ISO_WD}/splash.png" ]] && cp "${_ARCHBOOT_ISO_WD}/splash.png" "${_ARCHBOOT_ISO_EXT_DIR}/boot/syslinux/splash.png"
+[[ -e "${_ARCHBOOT_ISO_WD}/splash.png" ]] && cp -f "${_ARCHBOOT_ISO_WD}/splash.png" "${_ARCHBOOT_ISO_EXT_DIR}/boot/syslinux/splash.png"
 echo
 
 _rename_old_files() {
@@ -83,11 +105,16 @@ _rename_old_files() {
 _update_syslinux_iso_files() {
 	
 	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/syslinux"/*.{com,bin,c32} || true
-	cp "/usr/lib/syslinux"/*.{com,bin,c32} "${_ARCHBOOT_ISO_EXT_DIR}/boot/syslinux/"
+	cp -f "/usr/lib/syslinux"/*.{com,bin,c32} "${_ARCHBOOT_ISO_EXT_DIR}/boot/syslinux/"
+	
+}
+
+_update_syslinux_iso_config() {
 	
 	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/syslinux/syslinux.cfg" || true
 	
 	cat << EOF > "${_ARCHBOOT_ISO_EXT_DIR}/boot/syslinux/syslinux.cfg"
+
 SERIAL 0 38400
 DEFAULT vesamenu.c32
 PROMPT 0
@@ -127,6 +154,11 @@ For troubleshooting and other options press F2 key.
 ENDTEXT
 MENU LABEL Help
 
+EOF
+	
+	if [[ "${_REMOVE_x86_64}" != "1" ]]; then
+		cat << EOF >> "${_ARCHBOOT_ISO_EXT_DIR}/boot/syslinux/syslinux.cfg"
+
 LABEL arch64
 TEXT HELP
 Boot the Arch Linux (x86_64) archboot medium. 
@@ -147,6 +179,12 @@ LINUX /boot/vmlinuz_x86_64_lts
 APPEND gpt loglevel=7 rootdelay=10
 INITRD /boot/initramfs_x86_64.img
 
+EOF
+	fi
+	
+	if [[ "${_REMOVE_i686}" != "1" ]]; then
+		cat << EOF >> "${_ARCHBOOT_ISO_EXT_DIR}/boot/syslinux/syslinux.cfg"
+
 LABEL arch32
 TEXT HELP
 Boot the Arch Linux (i686) archboot medium. 
@@ -167,6 +205,11 @@ LINUX /boot/vmlinuz_i686_lts
 APPEND gpt loglevel=7 rootdelay=10
 INITRD /boot/initramfs_i686.img
 
+EOF
+	fi
+	
+	cat << EOF >> "${_ARCHBOOT_ISO_EXT_DIR}/boot/syslinux/syslinux.cfg"
+
 LABEL existing
 TEXT HELP
 Boot an existing operating system. Press TAB to edit the disk and partition
@@ -193,60 +236,89 @@ LABEL poweroff
 MENU LABEL Power Off
 COMBOOT poweroff.com
 
+EOF
+	
+	if [[ "${_REMOVE_x86_64}" != "1" ]]; then
+		cat << EOF >> "${_ARCHBOOT_ISO_EXT_DIR}/boot/syslinux/syslinux.cfg"
+
+ONTIMEOUT arch64
+
+EOF
+	elif [[ "${_REMOVE_x86_64}" == "1" ]] && [[ "${_REMOVE_i686}" != "1" ]]; then
+		cat << EOF >> "${_ARCHBOOT_ISO_EXT_DIR}/boot/syslinux/syslinux.cfg"
+
 ONTIMEOUT arch32
 
 EOF
+	fi
 	
 }
 
 _download_uefi_shell_tianocore() {
 	
-	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/"
+	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/"
+	
+	mv "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64.efi" "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64.efi" || true
+	mv "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64_old.efi" "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64_old.efi" || true
+	rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/" || true
+	echo
 	
 	## Download Tianocore UDK/EDK2 ShellBinPkg UEFI x86_64 "Full Shell" - For Spec. Ver. >=2.3 systems
 	
-	mv "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64.efi" "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64.efi.backup" || true
+	mv "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64.efi" "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64.efi.backup" || true
 	echo
 	
 	if [[ -e "${_ARCHBOOT_ISO_WD}/shellx64.efi" ]]; then
-		cp "${_ARCHBOOT_ISO_WD}/shellx64.efi" "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64.efi"
+		cp -f "${_ARCHBOOT_ISO_WD}/shellx64.efi" "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64.efi"
 		echo
 	else
-		curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64.efi" "https://edk2.svn.sourceforge.net/svnroot/edk2/trunk/edk2/ShellBinPkg/UefiShell/X64/Shell.efi" || true
+		curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64.efi" "https://edk2.svn.sourceforge.net/svnroot/edk2/trunk/edk2/ShellBinPkg/UefiShell/X64/Shell.efi" || true
 		echo
 		
-		if [[ ! "$(file "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64.efi" | grep 'executable')" ]]; then
-			rm -f "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64.efi" || true
-			mv "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64.efi.backup" "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64.efi" || true
+		if [[ ! "$(file "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64.efi" | grep 'executable')" ]]; then
+			rm -f "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64.efi" || true
+			mv "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64.efi.backup" "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64.efi" || true
 		fi
 	fi
 	
-	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64.efi.backup" || true
+	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64.efi.backup" || true
 	echo
 	
 	## Download Tianocore UDK/EDK2 EdkShellBinPkg UEFI x86_64 "Full Shell" - For Spec. Ver. <2.3 systems
 	
-	mv "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64_old.efi" "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64_old.efi.backup" || true
+	mv "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64_old.efi" "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64_old.efi.backup" || true
 	echo
 	
 	if [[ -e "${_ARCHBOOT_ISO_WD}/shellx64_old.efi" ]]; then
-		cp "${_ARCHBOOT_ISO_WD}/shellx64_old.efi" "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64_old.efi"
+		cp -f "${_ARCHBOOT_ISO_WD}/shellx64_old.efi" "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64_old.efi"
 		echo
 	else
-		curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64_old.efi" "https://edk2.svn.sourceforge.net/svnroot/edk2/trunk/edk2/EdkShellBinPkg/FullShell/X64/Shell_Full.efi" || true
+		curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64_old.efi" "https://edk2.svn.sourceforge.net/svnroot/edk2/trunk/edk2/EdkShellBinPkg/FullShell/X64/Shell_Full.efi" || true
 		echo
 		
-		if [[ ! "$(file "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64_old.efi" | grep 'executable')" ]]; then
-			rm -f "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64_old.efi" || true
-			mv "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64_old.efi.backup" "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64_old.efi" || true
+		if [[ ! "$(file "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64_old.efi" | grep 'executable')" ]]; then
+			rm -f "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64_old.efi" || true
+			mv "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64_old.efi.backup" "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64_old.efi" || true
 		fi
 	fi
 	
-	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/efi/shell/shellx64_old.efi.backup" || true
+	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/efi/tools/shellx64_old.efi.backup" || true
 	echo
 	
 }
 
+_update_uefi_refind_sourceforge() {
+	
+	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}/packages/" || true
+	
+	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/packages/refind-bin.zip" || true
+	echo
+	
+	## Download latest rEFInd bin archive from sourceforge
+	curl --verbose -f -C - --ftp-pasv --retry 3 --retry-delay 3 -o "${_ARCHBOOT_ISO_EXT_DIR}/packages/refind-bin.zip" -L "http://sourceforge.net/projects/refind/files/latest/download"
+	
+}
+
 _update_grub_uefi_arch_specific_iso_files() {
 	
 	[[ "${_UEFI_ARCH}" == "x86_64" ]] && _SPEC_UEFI_ARCH="x64"
@@ -290,7 +362,7 @@ EOF
 	
 	echo
 	
-	cp "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/grub_standalone_archboot.cfg" "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/grub.cfg"
+	cp -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/grub_standalone_archboot.cfg" "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/grub.cfg"
 	
 	__ARCHBOOT_ISO_WD="${PWD}/"
 	
@@ -303,7 +375,7 @@ EOF
 	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/grub.cfg"
 	
 	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}/efi/boot/"
-	cp "${grub_uefi_mp}/efi/boot/boot${_SPEC_UEFI_ARCH}.efi" "${_ARCHBOOT_ISO_EXT_DIR}/efi/boot/boot${_SPEC_UEFI_ARCH}.efi"
+	cp -f "${grub_uefi_mp}/efi/boot/boot${_SPEC_UEFI_ARCH}.efi" "${_ARCHBOOT_ISO_EXT_DIR}/efi/boot/boot${_SPEC_UEFI_ARCH}.efi"
 	
 	echo
 	
@@ -317,7 +389,6 @@ _update_grub_uefi_iso_files() {
 	grub_uefi_mp="$(mktemp -d /tmp/grub_uefi_mp.XXX)"
 	
 	rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub" || true
-	rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/efi/grub2" || true
 	echo
 	
 	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub"
@@ -360,14 +431,8 @@ _update_grub_uefi_iso_files() {
 	unset LOOP_DEVICE
 	echo
 	
-	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/efi/boot/grub.cfg" || true
-	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/grub.cfg" || true
-	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub//boot/grub/grub_archboot.cfg" || true
-	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub//boot/grub/grub_standalone_archboot.cfg" || true
-	echo
-	
 	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/fonts/"
-	cp "/usr/share/grub/unicode.pf2" "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/fonts/"
+	cp -f "/usr/share/grub/unicode.pf2" "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/fonts/"
 	echo
 	
 	rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/locale/" || true
@@ -382,6 +447,12 @@ _update_grub_uefi_iso_files() {
 		# fi
 	# done
 	
+}
+
+_update_grub_uefi_iso_config() {
+	
+	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/grub_archboot.cfg" || true
+	
 	cat << EOF > "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/grub_archboot.cfg"
 
 set _kernel_params="gpt loglevel=7"
@@ -457,6 +528,11 @@ insmod search_fs_file
 insmod linux
 insmod chain
 
+EOF
+	
+	if [[ "${_REMOVE_x86_64}" != "1" ]]; then
+		cat << EOF >> "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/grub_archboot.cfg"
+
 if [ cpuid -l ]; then
 
     menuentry "Arch Linux (x86_64) archboot" {
@@ -475,6 +551,12 @@ if [ cpuid -l ]; then
 
 fi
 
+EOF
+	fi
+	
+	if [[ "${_REMOVE_i686}" != "1" ]]; then
+		cat << EOF >> "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/grub_archboot.cfg"
+
 menuentry "Arch Linux (i686) archboot" {
     set gfxpayload="keep"
     set root="\${archboot}"
@@ -489,16 +571,21 @@ menuentry "Arch Linux LTS (i686) archboot" {
     initrd /boot/initramfs_i686.img
 }
 
+EOF
+	fi
+	
+	cat << EOF >> "${_ARCHBOOT_ISO_EXT_DIR}/boot/grub/grub_archboot.cfg"
+
 if [ "\${grub_platform}" == "efi" ]; then
 
     menuentry "UEFI \${_UEFI_ARCH} Shell 2.0 - For Spec. Ver. >=2.3 systems" {
         set root="\${archboot}"
-        chainloader /efi/shell/shell\${_SPEC_UEFI_ARCH}.efi
+        chainloader /efi/tools/shell\${_SPEC_UEFI_ARCH}.efi
     }
 
     menuentry "UEFI \${_UEFI_ARCH} Shell 1.0 - For Spec. Ver. <2.3 systems" {
         set root="\${archboot}"
-        chainloader /efi/shell/shell\${_SPEC_UEFI_ARCH}_old.efi
+        chainloader /efi/tools/shell\${_SPEC_UEFI_ARCH}_old.efi
     }
 
 fi
@@ -547,7 +634,7 @@ _update_arch_setup_initramfs() {
 		
 		cd "${_initramfs_ext}/"
 		
-		find . -print0 | bsdcpio -0oH newc | xz --check=crc32 --lzma2=dict=1MiB > "${_ARCHBOOT_ISO_WD}/${_initramfs_name}.img"
+		find . -print0 | bsdcpio -0oH newc | lzma > "${_ARCHBOOT_ISO_WD}/${_initramfs_name}.img"
 		echo
 		
 		rm -f "${_ARCHBOOT_ISO_EXT_DIR}/boot/${_initramfs_name}.img" || true
@@ -590,12 +677,6 @@ _download_pkgs() {
 
 _rename_old_files
 
-[[ "${_UPDATE_SYSLINUX}" == "1" ]] && _update_syslinux_iso_files
-
-[[ "${_UPDATE_UEFI_SHELL}" == "1" ]] && _download_uefi_shell_tianocore
-
-[[ "${_UPDATE_GRUB_UEFI}" == "1" ]] && _update_grub_uefi_iso_files
-
 [[ "${_REMOVE_i686}" == "1" ]] && _remove_i686_iso_files
 
 [[ "${_REMOVE_x86_64}" == "1" ]] && _remove_x86_64_iso_files
@@ -603,22 +684,38 @@ _rename_old_files
 if [[ "${_UPDATE_SETUP}" == "1" ]] && [[ -e "${_ARCHBOOT_ISO_WD}/setup" ]]; then
 	cd "${_ARCHBOOT_ISO_WD}/"
 	
-	## The old method I tried, mount -o ro -t iso9660 /dev/sr0 /src, mv /arch/setup /arch/setup.old, cp /src/arch/setup /arch/setup, umount /dev/sr0
-	# cp ${_ARCHBOOT_ISO_WD}/setup ${_ARCHBOOT_ISO_EXT_DIR}/arch/setup
+	mkdir -p "${_ARCHBOOT_ISO_EXT_DIR}/arch/" || true
 	
-	## Extracting using bsdtar, replacing /arch/setup and recompressing the iniramfs archive does not work. Archive format not compatible with initramfs format.
-	## Compressing using bsdcpio and using 'newc' archive format works, taken from falconindy's geninit program.
+	rm -f "${_ARCHBOOT_ISO_EXT_DIR}/arch/setup" || true
+	cp -f "${_ARCHBOOT_ISO_WD}/setup" "${_ARCHBOOT_ISO_EXT_DIR}/arch/setup"
 	
-	_initramfs_name="initramfs_x86_64"
-	_update_arch_setup_initramfs
+	echo
 	
-	_initramfs_name="initramfs_i686"
-	_update_arch_setup_initramfs
+	if [[ "${_REMOVE_x86_64}" != "1" ]]; then
+		_initramfs_name="initramfs_x86_64"
+		_update_arch_setup_initramfs
+	fi
+	
+	if [[ "${_REMOVE_i686}" != "1" ]]; then
+		_initramfs_name="initramfs_i686"
+		_update_arch_setup_initramfs
+	fi
 	
 	echo
 fi
 
-## Re-create the archboot ISO
+[[ "${_UPDATE_UEFI_SHELL}" == "1" ]] && _download_uefi_shell_tianocore
+
+[[ "${_UPDATE_UEFI_REFIND}" == "1" ]] && _update_uefi_refind_sourceforge
+
+[[ "${_UPDATE_SYSLINUX}" == "1" ]] && _update_syslinux_iso_files
+
+[[ "${_UPDATE_SYSLINUX_CONFIG}" == "1" ]] && _update_syslinux_iso_config
+
+[[ "${_UPDATE_GRUB_UEFI}" == "1" ]] && _update_grub_uefi_iso_files
+
+[[ "${_UPDATE_GRUB_UEFI_CONFIG}" == "1" ]] && _update_grub_uefi_iso_config
+
 cd "${_ARCHBOOT_ISO_WD}/"
 
 ## Generate the BIOS+UEFI+ISOHYBRID ISO image using xorriso (extra/libisoburn package) in mkisofs emulation mode
@@ -629,16 +726,13 @@ xorriso -as mkisofs \
 	-omit-version-number -allow-leading-dots \
 	-relaxed-filenames -allow-lowercase -allow-multidot \
 	-volid "ARCHBOOT" \
-	-p "prepared by ${_BASENAME}" \
+	-preparer "prepared by ${_BASENAME}" \
 	-eltorito-boot boot/syslinux/isolinux.bin \
 	-eltorito-catalog boot/syslinux/boot.cat \
 	-no-emul-boot -boot-load-size 4 -boot-info-table \
 	-eltorito-alt-boot --efi-boot boot/grub/grub_uefi_x86_64.bin -no-emul-boot \
 	-isohybrid-mbr /usr/lib/syslinux/isohdpfx.bin \
-	-output "${_ARCHBOOT_ISO_UPDATED_PATH}" "${_ARCHBOOT_ISO_EXT_DIR}/" > /dev/null 2>&1
-echo
-
-rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/"
+	-output "${_ARCHBOOT_ISO_UPDATED_PATH}" "${_ARCHBOOT_ISO_EXT_DIR}/" &> "/tmp/archboot_update_xorriso.log"
 echo
 
 set +x
@@ -647,6 +741,9 @@ if [[ -e "${_ARCHBOOT_ISO_UPDATED_PATH}" ]]; then
 	echo
 	echo "Updated iso has been saved at ${_ARCHBOOT_ISO_UPDATED_PATH} ."
 	echo
+	
+	rm -rf "${_ARCHBOOT_ISO_EXT_DIR}/"
+	echo
 else
 	echo
 	echo "No updated iso found at ${_ARCHBOOT_ISO_UPDATED_PATH} due to some error."
@@ -654,12 +751,16 @@ else
 	echo
 fi
 
-unset _UPDATE_SYSLINUX
-unset _UPDATE_UEFI_SHELL
-unset _UPDATE_GRUB_UEFI
-unset _UPDATE_SETUP
+
 unset _REMOVE_i686
 unset _REMOVE_x86_64
+unset _UPDATE_SETUP
+unset _UPDATE_UEFI_SHELL
+unset _UPDATE_UEFI_REFIND
+unset _UPDATE_SYSLINUX
+unset _UPDATE_SYSLINUX_CONFIG
+unset _UPDATE_GRUB_UEFI
+unset _UPDATE_GRUB_UEFI_CONFIG
 unset _ARCHBOOT_ISO_OLD_PATH
 unset _ARCHBOOT_ISO_WD
 unset _ARCHBOOT_ISO_OLD_NAME
